"""Update password_hash length

Revision ID: cfaa3c85d3d3
Revises: 0ba35b227f10
Create Date: 2022-03-22 15:23:58.888856

"""
import sqlalchemy as sa
from alembic import op

import fief

# revision identifiers, used by Alembic.
revision = "cfaa3c85d3d3"
down_revision = "0ba35b227f10"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "fief_users",
        "hashed_password",
        existing_type=sa.String(length=72),
        type_=sa.String(length=255),
        existing_nullable=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
